package scj.algorithm.parallel.nodetask.switching;

import scj.algorithm.parallel.DAParallelAlgorithm;
import scj.algorithm.parallel.FlatSearchParallelization;
import scj.algorithm.tree.leftside.FlatLeftTree;
import scj.algorithm.tree.leftside.FlatLeftTreeDirectBuild;
import scj.algorithm.tree.rightside.PIETree;
import scj.algorithm.twotrees.labeling.flat.FlatTreeCreatorLeftSideDirect;
import scj.algorithm.twotrees.labeling.flat.FlatTreeCreatorRightSideDirect;
import scj.input.DataTuple;
import scj.result.Result;
import scj.runtime.Executable;
import scj.runtime.RuntimeCalculator;

import java.util.Set;

public class SwitchByLevel extends DAParallelAlgorithm {

    @Override
    public void execute(final Set<DataTuple> n_a, final Set<DataTuple> n_b, Result result) {
        RuntimeCalculator rc = new RuntimeCalculator(this.getClass());

        final FlatLeftTreeDirectBuild t1 =
                rc.measure(new Executable<FlatLeftTreeDirectBuild>() {
                    public FlatLeftTreeDirectBuild execute() {
                        return new FlatTreeCreatorLeftSideDirect().getInnerFlatTree(n_a, sortOrder);
                    }
                }, "build prefix tree 1 -- build up phase");
        rc.measure(new Executable<FlatLeftTree>() {
            public FlatLeftTree execute() {
                t1.complete();
                return t1;
            }
        }, "build prefix tree 1 -- completion");
        LOGGER.debug("Prefix tree of data set 1: {}", t1);

        PIETree t2=
                rc.measure(new Executable<PIETree>() {
                    public PIETree execute() {
                        return new FlatTreeCreatorRightSideDirect().getOuterFlatTree(n_b, sortOrder);
                    }
                }, "build prefix tree 2");
        LOGGER.debug("Prefix tree of data set 2: {}", t2);

        LOGGER.info("{}", rc);

        //Do scj on it
        if(scj.evaluation.Executor.MODE == scj.evaluation.Executor.EXECUTION_MODE.Time) {
            FlatSearchParallelization search = new SwitchByLevelSearch(t1, t2, result, this.getThreads());
            search.search();
        }

    }
}

